clear

%% Define model variables
% Unknown variables
Xvar = {'p1','p2','p3','q0','q1','q2','q3','dp1','dp2','dp3'};
Ivar = {'p1','p2','p3'};
Dvar = {'dp1','dp2','dp3'};

% Fault variables
Fvar = {'fV1','fV2','fV3','fT1','fT2','fT3'};

% Known variables
Zvar = {'y1','y2','y3'};

%% Define model structure
% Each line represents a model relation and lists all involved variables. 

% Algebraic constraints in Xvar
relsX = {...
    {'q1','p1','p2','fV1'},... % e1
    {'q2','p2','p3','fV2'},... % e2
    {'q3','p3','fV3'},... % e3
    {'dp1','q0','q1','fT1'},... % e4
    {'dp2','q1','q2','fT2'},... % e5
    {'dp3','q2','q3','fT3'},... % e6
    {'p1','y1'},... % e7
    {'q2','y2'},... % e8
    {'q0','y3'},... % e9
    {},... % e10
    {},... % e11
    {},... % e12
};

% Dynamic constraints in Ivar, Dvar
relsDiff = {...
    {},... % e1
    {},... % e2
    {},... % e3
    {},... % e4
    {},... % e5
    {},... % e6
    {},... % e7
    {},... % e8
    {},... % e9
    {'dp1','p1'},... % e10
    {'dp2','p2'},... % e11
    {'dp3','p3'},... % e12
};


%% Define the incidence matrices based on the model structure

% Compute incidence matrix Xtot for the unknown variables Xvar
% A 1 represents an edge in Ex, a 2 an edge in Ei, and a 3 an edge in Ed
X = symbol2incidence(relsX, Xvar)>0;
[foo,pos]=ismember(Dvar,Xvar);
D = 0*X; D(:,pos)=symbol2incidence(relsDiff, Dvar)>0;
[foo,pos]=ismember(Ivar,Xvar);
I = 0*X; I(:,pos)=symbol2incidence(relsDiff, Ivar)>0;
Xtot = X + 2*I + 3*D;

F = symbol2incidence(relsX, Fvar)>0;
Z = symbol2incidence(relsX, Zvar)>0;

%% Build a structural model object
SM      = CreateSM(Xtot,F,Z,{},Xvar,Fvar,Zvar); 
SM.name = 'Three-tank system';

% remove temporary variables
clear D F I Xtot Z foo relsDiff Dvar Fvar Ivar X Xvar Zvar pos relsX

%% Plot Model
figure(1)
PlotSM(SM);

%% Compute isolability properties
Imix=IsolMatrix(SM,@computeMixed);
Ider=IsolMatrix(SM,@computeDeriv);
Iint=IsolMatrix(SM,@computeInteg);

%% Plot results

% Mixed causality
figure(2)
set(gcf,'color','white')
spy(Imix)
set(get(gcf,'CurrentAxes'),'XTickLabel',{'', SM.f{:}})
set(get(gcf,'CurrentAxes'),'YTickLabel',{'', SM.f{:}})
xlabel('Diagnosed fault')
ylabel('Injected fault')
title('Mixed causality')

% Derivative causality
figure(3)
set(gcf,'color','white')
spy(Ider)
set(get(gcf,'CurrentAxes'),'XTickLabel',{'', SM.f{:}})
set(get(gcf,'CurrentAxes'),'YTickLabel',{'', SM.f{:}})
xlabel('Diagnosed fault')
ylabel('Injected fault')
title('Derivative causality')

% Integral causality
figure(4)
set(gcf,'color','white')
spy(Iint)
set(get(gcf,'CurrentAxes'),'XTickLabel',{'', SM.f{:}})
set(get(gcf,'CurrentAxes'),'YTickLabel',{'', SM.f{:}})
xlabel('Diagnosed fault')
ylabel('Injected fault')
title('Integral causality')

